#ifndef COMPLEX_H
#define COMPLEX_H

#include <cmath>
#include <iostream>

class complex_number
{
public:
    /*
     * takes in the real and imaginary
     * components values e.g. z = a + bi
     */
    complex_number(double a, double b)
        : m_a(a), m_b(b)
    {
    }

    complex_number()
        : m_a(1.0), m_b(1.0)
    {
    }
    
    /*
     * complex multiplication
     */
    complex_number operator*(const complex_number& rhs) const
    {
        complex_number result;

        result.m_a = m_a * rhs.m_a - m_b * rhs.m_b;
        result.m_b = m_a * rhs.m_b + m_b * rhs.m_a;
        
        return result;
    }

    /*
     * complex addition
     */
    complex_number operator+(const complex_number& rhs) const
    {
        complex_number result;

        result.m_a = m_a + rhs.m_a;
        result.m_b = m_b + rhs.m_b;

        return result;
    }

    /*
     * complex subtraction
     */
    complex_number operator-(const complex_number& rhs) const
    {
        complex_number result;

        result.m_a = m_a - rhs.m_a;
        result.m_b = m_b - rhs.m_b;

        return result;
    }

    /*
     * length
     */
    double length() const
    {
        return sqrt(m_a * m_a + m_b * m_b);
    }

    /*
     * get real and imaginary components
     */
    void get_components(double &a, double &b) const
    {
        a = m_a;
        b = m_b;
        //return m_a;
    }

    /*
     * set real and imaginary components
     */
    void set_components(double a, double b) 
    {
        m_a = a;
        m_b = b;
    }

    void print()
    {
        std::cout << "(" << m_a << "," << m_b << ")" << std:: endl;
    }

private:
    double m_a, m_b; // real and imaginary component;
    
}; // class complex_number

#endif
