//
//		      Programming Assignment #1 for CS 130 
//
//	                               PUT YOUR NAME HERE
//		
//		
//
/***************************************************************************/
#include <iostream>
#include <stack>
#include <GL/glut.h>

#include "complex.h"

using namespace std;

#define WIDTH 500   //width of window
#define HEIGHT 500  //height of window

struct rectangle
{
    double x0, y0;
    double x1, y1;  
}; // struct box

/*
 * boundary of the complex plane corresponding to our viewport
 */
#define INITIAL_CBOUND_X0 -2.0
#define INITIAL_CBOUND_X1 0.50
#define INITIAL_CBOUND_Y0 -1.25
#define INITIAL_CBOUND_Y1 1.25

#define MAX_ITERATIONS 40

// globals
int x_last; //x-coordinate of last mouse click
int y_last; //y-coordinate of last mouse click

int counter = 0;

stack<rectangle> zoom_history;  // history of complex boundary
rectangle new_complex_boundary; // new complex boundary
rectangle complex_boundary;	// current complex boundary
rectangle viewport;


// function prototypes
void init();
void write_pixel( int, int );
void display_callback();
void mouse_callback( int, int, int, int );
void keyboard_callback( unsigned char, int, int );
void idle_callback();

// init
// creates a new window
void init (int argc, char *argv[])
{
   // initialize view port
   viewport.x0 = 0.0; viewport.y0 = 0.0;
   viewport.x1 = WIDTH; viewport.y1 = HEIGHT;

   // initialize complex boundary
   complex_boundary.x0 = INITIAL_CBOUND_X0;
   complex_boundary.y0 = INITIAL_CBOUND_Y0;
   complex_boundary.x1 = INITIAL_CBOUND_X1;
   complex_boundary.y1 = INITIAL_CBOUND_Y1;


   glutInit( &argc, argv );                // initialize GLUT
   glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE );
   glutInitWindowSize( WIDTH, HEIGHT );
   glutCreateWindow( "Riverside Graphics" );
   glutDisplayFunc( display_callback );    // set display callback function
   glutMouseFunc( mouse_callback );        // set mouse callback function
   glutKeyboardFunc( keyboard_callback );  // set keyboard callback function
   glClearColor( 0.0, 0.0, 0.0, 0 );       // background color = black
   glOrtho( 0, WIDTH, 0, HEIGHT, -1, 1 );  // orthogonal projection
}

// write_pixel
// colors the pixel at x, y
void write_pixel( int x, int y, double r, double g, double b)
{
   glColor3f( r, g, b );
   glBegin( GL_POINTS );
   glVertex3i( x, y, 0 );
   glEnd ();
}


/*
 * takes the position of a pixel defined in the viewport space and maps it to a point
 * in complex space. 
 */
complex_number viewport_to_complex_plane(
    const rectangle& viewport, const rectangle& complex_boundary,
    int x, int y
    )
{
    double viewport_width =  abs(viewport.x1 - viewport.x0);
    double viewport_height = abs(viewport.y1 - viewport.y0);
    
    double complex_boundary_width = abs(complex_boundary.x1 - complex_boundary.x0);
    double complex_boundary_height = abs(complex_boundary.y1 - complex_boundary.y0);

    complex_number z;

    z.set_components(
            x / viewport_width * complex_boundary_width + complex_boundary.x0,
            y / viewport_height * complex_boundary_height + complex_boundary.y0
        );

    return z;
}   

// display_callback
// called when the window needs to be redisplayed
void display_callback ()
{
   glClear( GL_COLOR_BUFFER_BIT );

    for (int i = 0; i < WIDTH; ++i)
    {
        for (int j = 0; j < HEIGHT; ++j)
        {
            // maps the pixel (i, j) to a coordinate in the complex plane
            complex_number z0 = viewport_to_complex_plane(viewport, complex_boundary, i, j);

            /*
             *
             * YOUR CODE HERE
             *
             */
        }
    }
   
    glutSwapBuffers();
}


// mouse_callback
// called when a mouse event occurs
void mouse_callback ( int button, int button_state, int x, int y )
{
   // the "y" value here is 0 at top & gets larger as you go down
   // adjust "y" so 0 is at the bottom of window
   y = HEIGHT - y;

   // store point if user just depressed the left button
   if ( button == GLUT_LEFT_BUTTON && button_state == GLUT_DOWN)
   {
      counter++;
      if (counter == 1)
      {
         complex_number z = viewport_to_complex_plane(viewport, complex_boundary, x, y);
         z.get_components( new_complex_boundary.x0, new_complex_boundary.y0 );
      }
      else if (counter == 2)
      {
         complex_number z = viewport_to_complex_plane(viewport, complex_boundary, x, y);
         z.get_components( new_complex_boundary.x1, new_complex_boundary.y1 );

         if (new_complex_boundary.x0 > new_complex_boundary.x1) {
            double swap = new_complex_boundary.x1;
            new_complex_boundary.x1 = new_complex_boundary.x0;
            new_complex_boundary.x0 = swap;
         }
         if (new_complex_boundary.y0 > new_complex_boundary.y1) {
            double swap = new_complex_boundary.y1;
            new_complex_boundary.y1 = new_complex_boundary.y0;
            new_complex_boundary.y0 = swap;
         }

         // push current complex boundary into stack
         zoom_history.push( complex_boundary );

         // update new complex boundary
         complex_boundary.x0 = new_complex_boundary.x0;
         complex_boundary.y0 = new_complex_boundary.y0;
         complex_boundary.x1 = new_complex_boundary.x1;
         complex_boundary.y1 = new_complex_boundary.y1;

         counter = 0;
      }

      // save pixel just clicked
      x_last = x;
      y_last = y;
      glutIdleFunc( idle_callback );
   }
}


// keyboard_callback
// called when keyboard event occurs
void keyboard_callback( unsigned char key, int x, int y )
{
   switch ( key )
   {
      // When escape is pressed, exit the program
      case 27:
         exit (0);
         break;

      // When 'c' is pressed, clear the screen
      case 'c':
         cout << "'c' was pressed\n";
         glClear( GL_COLOR_BUFFER_BIT );
         glutSwapBuffers();
         break;

      // When 'z' is pressed, go back to last complex boundary
      case 'z':
         new_complex_boundary = zoom_history.top();
         zoom_history.pop();

         complex_boundary.x0 = new_complex_boundary.x0;
         complex_boundary.y0 = new_complex_boundary.y0;
         complex_boundary.x1 = new_complex_boundary.x1;
         complex_boundary.y1 = new_complex_boundary.y1;

	 glutIdleFunc( idle_callback );
         break;

      default:
         break;
   }
}

// idle_callback
// called whenever the window is idling
void idle_callback()
{
   glutPostRedisplay();
   glutIdleFunc( NULL );
}

// main
// creates a new window and enters the main glut loop
int main (int argc, char *argv[])
{
   init( argc, argv );
   glutMainLoop();

   return( 0 );
}
